//****************************************************************************** //
// $Id: $
// 
// Original Author: Gordon Roxburgh // Creation Date: 9th September 2016
// $Author: $
// $Date: $
// $Revision: $
//
//*****************************************************************************
//
// Description: Example code showing how to do spectral stitching with the NI VST.
//
// Copyright (c) 2005-20016, Aeroflex International Ltd.
//
//
//*****************************************************************************

#include "stdafx.h"
#include "niRFSG.h"
#include "niRFSA.h"
#include "afVstWrapper.h"
#include <stdio.h>
#include <vector>
#include <fstream>
#include <string>
#include <iostream>
#include <direct.h>
#include <math.h>
#include "afSpectrumDll.h"
#include "afSpectrumDllDefs.h"

// The real and imaginary parts of this complex data array correspond to the in-phase (I) and quadrature-phase (Q) data,
// respectively. To calculate the instantaneous power of a sampled I/Q point, use the equation (I^2 + Q^2)/2R, where R is
// the input impedance in ohms (O). For NI RF vector signal analyzers, R = 50 O. 
// For use with analysis library a fixed level correction defined below can account for the W -> nW conversion of the IQ.
// 10 * log10(1000.0 /* W -> mW */ / (2 /* See NI help file */ * 50.0 /* input impedance - see NI help file */));
static const double VstLevelCorrection = 10 * log10(1000.0 / (2 * 50.0 ));

typedef enum {VST_5644R, VST_5645R, VST_5646R} model_t;

typedef enum { LO_Above, LO_Below, LO_AUTO, LO_None } loPosition_t;

static const double MaximumFrequency = 6e9;

static const double MinimumFrequencyRegion1 = 65e6;
static const double MaximumFrequencyRegion1  = 109e6;
static const double MinimumFrequencyRegion2_5644R_5645R = 109e6;
static const double MaximumFrequencyRegion2_5644R_5645R = 6e9;
static const double MinimumFrequencyRegion2_5646R = 109e6;
static const double MaximumFrequencyRegion2_5646R = 200e6;
static const double MinimumFrequencyRegion3_5646R = 200e6;
static const double MaximumFrequencyRegion3_5646R = 6e9;
static const double MaximumBandwidthRegion1_5644R_5645R = 10e6;
static const double MaximumBandwidthRegion2_5644R_5645R = 27e6;

static const double MaximumBandwidthRegion1_5646R = 10e6;
static const double MaximumBandwidthRegion2_5646R = 27e6;
static const double MaximumBandwidthRegion3_5646R = 38e6;

//****************************************************************************** 
// Function          : DumpXY
// Globals Used      : None
// Globals Modified  : None
// Description       : Writes out the xData, yData into a csv file.
//******************************************************************************
void DumpXY(char* fileName, double* xData, double* yData, int NumberOfSamples)
{
	  std::ofstream myfile;	  

      myfile.open (fileName);     
      myfile << "X,Y\n";
      
	  for( int index = 0; index < NumberOfSamples; index++)
	  {		 
		  myfile << xData[index] << "," << yData[index] <<  "\n";
	  }
	  
      myfile.close();     

	  std::cout << "File Written : " << fileName << std::endl;
}

//****************************************************************************** 
// Function          : GetStitchingBandwidth
// Globals Used      : None
// Globals Modified  : None
// Description       : Returns the correct LO offset to use when stitching based
// on the capture span, centre frequency and VST model.
//******************************************************************************
ViReal64 GetLoOffset(double Centrefrequency, double CaptureSpan, model_t Model, loPosition_t LoPosition)
{
	ViReal64 LoOffset = CaptureSpan + 2e6;
	double startFreq = Centrefrequency - (CaptureSpan /2);
	double stopFreq = Centrefrequency + (CaptureSpan /2);

	if ( LoPosition == LO_Above)
	{
		return LoOffset;
	}
	else if ( LoPosition == LO_Below)
	{
		return LoOffset * -1;
	}
	else if ( LoPosition == LO_None)
	{
		return 0;
	}
	else if ( LoPosition == LO_AUTO)
	{

		if ( Model == VST_5644R || Model == VST_5645R )
		{
			// Determine Correct LO offset value;
			if ( Centrefrequency + LoOffset > MaximumFrequency ) 
			{
				// LO will be out of range so set LO below.
				return LoOffset * -1;			
			}
			else if ( startFreq  >= MinimumFrequencyRegion1 && startFreq <= MaximumFrequencyRegion1) // Starts in Region 1.
			{
				// We always try to place the LO to the right of the actual spectrum that we will use, which means, in most cases VST
				// LO frequency > RF Center Frequency. However, if the segment starts in one region, and right-side placement puts the
				// LO in a different region, we will use left-side placement.
				if ( Centrefrequency + LoOffset > MaximumFrequencyRegion1)
				{
					return LoOffset * -1;
				}
				else
				{
					return LoOffset;
				}
			}
			else
			{
				// Default to always settting the LO above which means for other cases this will be ok.
				return LoOffset;
			}					
		}
		else if ( Model == VST_5646R )
		{
			// Determine Correct LO offset value;
			if ( Centrefrequency + LoOffset > MaximumFrequency ) 
			{
				// LO will be out of range so set LO below.
				return LoOffset * -1;			
			}
			else if ( startFreq  >= MinimumFrequencyRegion1 && startFreq <= MaximumFrequencyRegion1) // Starts in Region 1.
			{
				// We always try to place the LO to the right of the actual spectrum that we will use, which means, in most cases VST
				// LO frequency > RF Center Frequency. However, if the segment starts in one region, and right-side placement puts the
				// LO in a different region, we will use left-side placement.
				if ( Centrefrequency + LoOffset > MaximumFrequencyRegion1)
				{
					return LoOffset * -1;
				}
				else
				{
					return LoOffset;
				}
			}
			else if ( startFreq  >= MinimumFrequencyRegion2_5646R && startFreq <= MaximumFrequencyRegion2_5646R) // Starts in Region 2.
			{
				// We always try to place the LO to the right of the actual spectrum that we will use, which means, in most cases VST
				// LO frequency > RF Center Frequency. However, if the segment starts in one region, and right-side placement puts the
				// LO in a different region, we will use left-side placement.
				if ( Centrefrequency + LoOffset > MaximumFrequencyRegion2_5646R)
				{
					return LoOffset * -1;
				}
				else
				{
					return LoOffset;
				}
			}
			else // We can assume that we're in region 3 here and that LO above will not be over the 6GHz limit
			{
				// Default to always settting the LO above which means for other cases this will be ok.
				return LoOffset;
			}					
		}	
	}
}

//****************************************************************************** 
// Function          : ErrorCheckRFSA
// Globals Used      : None
// Globals Modified  : None
// Description       : Using a RFSA VISA session and a status code the status
// code is checked to see if it is either no error, an error or a warning.
//******************************************************************************
void ErrorCheckRFSA(ViSession Vsa, ViStatus Status)
{
	ViChar errMsg[256];

	if ( Status > 0 ) // Warning
	{		
		// Retrieve the error message based on the status code.
		niRFSA_error_message(Vsa, Status, errMsg);

		std::cout << "Warning : ";

		// Loop through the array of chars printing them out to the STDOUT.
		for( int i = 0 ; i < strlen(errMsg) ; i ++ )
		{
			std::cout << errMsg[i]; 
		}
		std::cout << std::endl;		
	}
	else if ( Status < 0 ) // Error
	{
		ViChar errMsg[256];

		// Retrieve the error message based on the status code.
		niRFSA_error_message(Vsa, Status, errMsg);

		std::cout << "Error : ";			

		// Loop through the array of chars printing them out to the STDOUT.
		for( int i = 0 ; i < strlen(errMsg) ; i ++ )
		{
			std::cout << errMsg[i]; 
		}
		std::cout << std::endl;		
	}	
}

//****************************************************************************** 
// Function          : ErrorCheckRFSG
// Globals Used      : None
// Globals Modified  : None
// Description       : Using a RFSG VISA session and a status code the status
// code is checked to see if it is either no error, an error or a warning.
//******************************************************************************
void ErrorCheckRFSG(ViSession Vsg, ViStatus Status)
{
	ViChar errMsg[256];

	if ( Status > 0 ) // Warning
	{
		// Retrieve the error message based on the status code.
		niRFSG_error_message(Vsg, Status, errMsg);

		std::cout << "Warning : ";

		// Loop through the array of chars printing them out to the STDOUT.
		for( int i = 0 ; i < strlen(errMsg) ; i ++ )
		{
			std::cout << errMsg[i]; 
		}
		std::cout << std::endl;		
	}
	else if ( Status < 0 ) // Error
	{		
		// Retrieve the error message based on the status code.
		niRFSG_error_message(Vsg, Status, errMsg);

		std::cout << "Error : ";			

		// Loop through the array of chars printing them out to the STDOUT.
		for( int i = 0 ; i < strlen(errMsg) ; i ++ )
		{
			std::cout << errMsg[i]; 
		}
		std::cout << std::endl;		
	}	

	//delete errMsg;
}

//****************************************************************************** 
// Function          : ErrorCheckWrapper
// Globals Used      : None
// Globals Modified  : None
// Description       : Using a status code generated from API calls to the afVSTWrapper
// the code is checked to see if it is either no error, an error or a warning occured.
//******************************************************************************
void ErrorCheckWrapper(long Status)
{
	std::string errorMessage;

	if ( Status < 0 ) // Error
	{		
		switch(Status)
		{
		case afVstWrapper_eUnknownError:
			{
				errorMessage = "Unknown error";
				break;
			}
		case afVstWrapper_eFailedToLoadRFSGLibrary:
			{
				errorMessage = "The RFSG LIbrary could not be loaded";
				break;
			}
		case afVstWrapper_eFailedToLoadRFSALibrary:
			{
				errorMessage = "The RFSA LIbrary could not be loaded";
				break;
			}
		case afVstWrapper_eFailedToQuerySerialNumber:
			{
				errorMessage = "An error occurred while attempting to read the VST serial number";
				break;
			}
		case afVstWrapper_eFailedToConnectToLicenseServer:
			{
				errorMessage = "A connection to the license service could not be created";
				break;
			}
		case afVstWrapper_eFailedToQueryLicenseServer:
			{
				errorMessage = "An error occurred while querying the license service for password details";
				break;
			}
		case afVstWrapper_eFailedToSignIQ:
			{
				errorMessage = "An error occurred while creating the key/tag for the IQ data";
				break;
			}
		case afVstWrapper_eNoPasswordForOption:
			{
				errorMessage = "No password is available for the specified option";
				break;
			}
		case afVstWrapper_eRequiredOptionNotEnabled:
			{
				errorMessage = "An option required to load the AIQ file is not enabled";
				break;
			}
		case afVstWrapper_eAiqFileCreatedWithWrongIQCreatorVersion:
			{
				errorMessage = "The AIQ file is using an obsolete format";
				break;
			}
		case afVstWrapper_eAiqFileOpenError:
			{
				errorMessage = "An error occurred while opening the AIQ file";
				break;
			}
		case afVstWrapper_eAiqFileReadError:
			{
				errorMessage = "An error occurred while reading the AIQ file";
				break;
			}
		case afVstWrapper_eAiqFileFormatError:
			{
				errorMessage = "The AIQ file contains data in an invalid format";
				break;
			}
		case afVstWrapper_eAiqFileChecksumError:
			{
				errorMessage = "The AIQ file checksum does not match the file contents";
				break;
			}
		case afVstWrapper_eAiqFileLoadFailure:
			{
				errorMessage = "An unexpected error occurred while loading the AIQ file";
				break;
			}
		case afVstWrapper_eTooManyIQSamples:
			{
				errorMessage = "Too many IQ samples";
				break;
			}
		case afVstWrapper_eMemAllocFailure:
			{
				errorMessage = "Memory allocation error";
				break;
			}
		case afVstWrapper_eMarkerBufferTooSmall:
			{
				errorMessage = "The supplied marker buffer is too small";
				break;
			}
		case afVstWrapper_eFunctionNotImplemented:
			{
				errorMessage = "A required function is not implemented in the RFSA or RFSG driver - check correct version installed.";
				break;
			}
		default:
			{
				errorMessage = "Unknown Error Code";
			}
		}

		std::cout << "Error : " << errorMessage.c_str() << std::endl;
	}		
}


//****************************************************************************** 
// Function          : BootRFSA
// Globals Used      : None
// Globals Modified  : None
// Description       : Boots the RFSA within the VST and returns a reference to
// the RFSA.
//******************************************************************************
unsigned long BootRFSA()
{
	ViStatus status;
	ViSession vsa = VI_NULL;
	ViReal64 referenceClockRate = 10e6; // 10 MHz ( VST only supports 10 MHz )

	// Initiate VSA session		
	status = niRFSA_init("RIO0", VI_TRUE, VI_TRUE, &vsa);					
	ErrorCheckRFSA(vsa,status);

	// Configure 10MHz reference
	status = niRFSA_ConfigureRefClock(vsa, NIRFSA_VAL_ONBOARD_CLOCK_STR, referenceClockRate);
	ErrorCheckRFSA(vsa,status);

	// Commit the settings to make clock settings active
	status = niRFSA_Commit(vsa);
	ErrorCheckRFSA(vsa,status);

	std::cout << "VST - RFSA Booted and Configured for 10 MHz reference clock. : " << std::endl;			

	return vsa;
}

//****************************************************************************** 
// Function          : BootRFSG
// Globals Used      : None
// Globals Modified  : None
// Description       : Boots the RFSG within the VST and returns a reference to
// the RFSG.
//******************************************************************************
unsigned long BootRFSG()
{
	ViStatus status;
	ViSession vsg = VI_NULL;
	ViReal64 referenceClockRate = 10e6; // 10 MHz ( VST only supports 10 MHz )

	// Initiate VSG session
	status = niRFSG_init("RIO0", VI_TRUE, VI_TRUE, &vsg);
	ErrorCheckRFSG(vsg,status);

	// Configure 10MHz reference
	status = niRFSG_ConfigureRefClock(vsg, NIRFSG_VAL_ONBOARD_CLOCK_STR, referenceClockRate); // onboard reference
	ErrorCheckRFSG(vsg,status);

	// Export reference clock to REF out connector (if required)
	status = niRFSG_ExportSignal(vsg, NIRFSG_VAL_REF_CLOCK, "", NIRFSG_VAL_REF_OUT_STR);
	ErrorCheckRFSG(vsg,status);

	// Commit the settings to make clock settings active
	status = niRFSG_Commit(vsg);
	ErrorCheckRFSG(vsg,status);

	std::cout << "VST - RFSG Booted and Configured for 10 MHz reference clock. : " << std::endl;		

	return vsg;
}

//****************************************************************************** 
// Function          : BootRFSG
// Globals Used      : None
// Globals Modified  : None
// Description       : Boots the RFSG within the VST and returns a reference to
// the RFSG.
//******************************************************************************
void CloseRFSA(ViSession vsa)
{
	ViStatus status;

	// Close session
	status = niRFSA_close(vsa);
	ErrorCheckRFSA(vsa,status);
}

//****************************************************************************** 
// Function          : BootRFSG
// Globals Used      : None
// Globals Modified  : None
// Description       : Boots the RFSG within the VST and returns a reference to
// the RFSG.
//******************************************************************************
void CloseRFSG(ViSession vsg)
{
	ViStatus status;

	// Close session
	status = niRFSG_close(vsg);
	ErrorCheckRFSG(vsg,status);
}

//****************************************************************************** 
// Function          : CloseWLAN
// Globals Used      : None
// Globals Modified  : None
// Description       : 
//******************************************************************************
void CloseSpectrum(afSpectrumDllObjectID SpectrumObjectID)
{
	std::cout << "Destroying Spectrum Analysis Object." << std::endl;
	long status = afSpectrumDll_DestroyObject( SpectrumObjectID);
	
	if (status != 0 )
	{
		std::cout << "An error occured shutting the Spectrum Analysis Library Down" << std::endl;
	}
}

//****************************************************************************** 
// Function          : BootRFSG
// Globals Used      : None
// Globals Modified  : None
// Description       : Generates a CW tone at a given frequency and level.
//******************************************************************************
void PlayAiqWaveform(ViSession vsg, ViConstString fileNameAiq, ViReal64 frequency, ViReal64 level)
{
	ViStatus status;
	ViBoolean done;
	ViReal64 headRoom = 0;
	ViReal64 samplingRate;
	ViReal64 crestFactor;
	ViReal64 peakPowerAdjust;
	ViReal64 averagePowerAdjust;
	ViInt32 markerBufferLength = 256;
	ViInt32 fileMarkerCount;
	afRFSG_MarkerDetails_t *markerBuffer = new afRFSG_MarkerDetails_t[markerBufferLength];
	char* drive = new char[256];
	char* folder = new char[256];
	char* filenameNoextension = new char[256];
	char* extension = new char[16];
		
	// Aborts generation & put VSG in configuration mode
	status = niRFSG_Abort(vsg);
	ErrorCheckRFSA(vsg,status);

	//
	// Configure all settings that require the VSG to be in configuration mode
	//

	// RMS mode will automatically rescale ARB files to maximise DAC dynamic range.
	// PEAK mode will treat the RF level as being the peak value, so the RMS value will usually be lower.  ARB files are not re-scaled.
	status = niRFSG_ConfigurePowerLevelType(vsg, NIRFSG_VAL_AVERAGE_POWER);
	ErrorCheckRFSA(vsg,status);

	// Clear All wavefroms so we don't get an error if the waveform is already loaded.
	status = niRFSG_ClearAllArbWaveforms (vsg);
	ErrorCheckRFSA(vsg,status);	

	// Remove the path and extension from the file so that a name can be associated with the IQ on the VST.
	_splitpath(fileNameAiq,drive,folder,filenameNoextension,extension);
	
	// Load the .aiq file using the wrapper and set the sample rate and on board filename.
	status = afVstWrapper_RFSG_LoadAiqFile(vsg, VI_FALSE ,fileNameAiq,filenameNoextension,headRoom,&samplingRate,&crestFactor, &peakPowerAdjust, &averagePowerAdjust,markerBuffer,markerBufferLength,&fileMarkerCount);	
	ErrorCheckWrapper(status);

	// Now Auto-Generate Marker Script
	//std::cout << "script wlan11ACExample4" << std::endl;
	//std::cout << "" << std::endl;
	//std::cout << "generate " << filenameNoextension;	

	// Set ARB playback sample rate
	status = niRFSG_SetAttributeViReal64(vsg, VI_NULL, NIRFSG_ATTR_IQ_RATE, samplingRate);
	ErrorCheckRFSA(vsg,status);

	// Set generation mode to ARB Waveform.
	status = niRFSG_ConfigureGenerationMode(vsg, NIRFSG_VAL_ARB_WAVEFORM);
	ErrorCheckRFSA(vsg,status);

	// Configure the ARB to playback continuously or a finite numberof times.
	status = niRFSG_SetAttributeViBoolean(vsg, VI_NULL, NIRFSG_ATTR_ARB_WAVEFORM_REPEAT_COUNT_IS_FINITE, VI_FALSE); // VI_FALSE for continuous playback VI_TRUE to play back n-times.
	ErrorCheckRFSA(vsg,status);

	// Configure number of times ARB file is played
	status = niRFSG_SetAttributeViInt32(vsg, VI_NULL, NIRFSG_ATTR_ARB_WAVEFORM_REPEAT_COUNT, 1000); 
	ErrorCheckRFSA(vsg,status);

	// Disable start trigger, so generation will commence immediately after _Initiate() called
	status = niRFSG_DisableStartTrigger(vsg);
	ErrorCheckRFSA(vsg,status);

	// Configure frequency & level using higher level function
	status = niRFSG_ConfigureRF(vsg, frequency, level + averagePowerAdjust);
	ErrorCheckRFSA(vsg,status);

	// Enable output
	status = niRFSG_ConfigureOutputEnabled(vsg, VI_TRUE);
	ErrorCheckRFSA(vsg,status);

	// Enable software start trigger
	status = niRFSG_ConfigureSoftwareStartTrigger(vsg);
	ErrorCheckRFSA(vsg,status);

	// If more than one waveform is loaded you require to select the correct file.
	status = niRFSG_SelectArbWaveform(vsg,filenameNoextension);
	ErrorCheckRFSA(vsg,status);

	// Check what waveform we're actually using:
	ViChar* selectedWaveform = new ViChar[256];

	status = niRFSG_GetAttributeViString(vsg,VI_NULL, NIRFSG_ATTR_ARB_SELECTED_WAVEFORM,256,selectedWaveform);
    ErrorCheckRFSA(vsg,status);

	// Initiate generation (VSG now in generation mode)
	status = niRFSG_Initiate(vsg);
	ErrorCheckRFSA(vsg,status);

	// Check Generation Status. ( returns 0 (generation active) )
	status = niRFSG_CheckGenerationStatus(vsg, &done); 
	ErrorCheckRFSA(vsg,status);

	// VSG now in generation mode but awaiting a software trigger to commence ARB playback
	status = niRFSG_SendSoftwareEdgeTrigger(vsg, NIRFSG_VAL_START_TRIGGER, ""); // send software trigger
	ErrorCheckRFSA(vsg,status);

	// Deallocate allocated memory.
	delete(markerBuffer);
	delete(drive);
	delete(folder);
	delete(filenameNoextension);
	delete(extension);
}

//****************************************************************************** 
// Function          : Capture
// Globals Used      : None
// Globals Modified  : None
// Description       : Captures IQ for a specified dwell time at a specified frequency and input level at a specified sample rate.
//******************************************************************************
void Capture(ViSession vsa, ViReal64 CarrierFrequency, ViReal64 ReferenceLevel, int NumSamples, ViReal64 SampleRate, loPosition_t LOPosition, double Span, model_t Model, float* iBuffer, float* qBuffer, double* Key, double* Tag )
{
	ViStatus status;

	//
	// Configure VST for capture
	//
	status = niRFSA_ConfigureAcquisitionType(vsa, NIRFSA_VAL_IQ); // IQ capture mode
	ErrorCheckRFSA(vsa,status);

	status = niRFSA_ConfigureIQCarrierFrequency(vsa, VI_NULL, CarrierFrequency); // Frequency
	ErrorCheckRFSA(vsa,status);

	status = niRFSA_ConfigureReferenceLevel(vsa, VI_NULL, ReferenceLevel); // Reference level
	ErrorCheckRFSA(vsa,status);

	// Work out what the LO offset should be.
	ViReal64 LoOffset = GetLoOffset(CarrierFrequency,Span, Model, LOPosition);
	std::cout << "LO Offset : " << LoOffset << " Hz" <<std::endl;

	// Set the LO Offset.
	status = niRFSA_SetAttributeViReal64(vsa, VI_NULL, NIRFSA_ATTR_DOWNCONVERTER_FREQUENCY_OFFSET, LoOffset);
	ErrorCheckRFSA(vsa,status);					
	
	// Set sample rate to that required by the analysis library for the selected measurements
	status = niRFSA_ConfigureIQRate(vsa, VI_NULL, SampleRate);
	ErrorCheckRFSA(vsa,status);
	
	status = niRFSA_ConfigureNumberOfSamples(vsa, VI_NULL, VI_TRUE, NumSamples); // Number of samples
	ErrorCheckRFSA(vsa,status);

	status = niRFSA_ConfigureNumberOfRecords(vsa, VI_NULL, VI_TRUE, 1); // Number of records
	ErrorCheckRFSA(vsa,status);

	status = niRFSA_DisableStartTrigger(vsa); // immediate trigger
	ErrorCheckRFSA(vsa,status);

	status = niRFSA_Initiate(vsa); // start capture
	ErrorCheckRFSA(vsa,status);

	if(status == 0)
	{
		// Fetch IQ data
		NIComplexNumberF32 *capturedIQ = new NIComplexNumberF32[NumSamples];

		status = niRFSA_SetAttributeViInt32(vsa, VI_NULL, NIRFSA_ATTR_FETCH_RELATIVE_TO, NIRFSA_VAL_FIRST_SAMPLE);
		ErrorCheckRFSA(vsa,status);

		niRFSA_wfmInfo captureInfo;
		ViReal64 timeout = 1.0;
		
		// Use AfVstWrapper to fetch IQ samples instead of RFSA niRFSA_FetchIQSingleRecordComplexF32(...) method call so that Tag and Key
		// are generated ( if license is fitted ) so that the signal can be analysed with the Cobham WLAN analysis library.
		status = afVstWrapper_RFSA_FetchIQSingleRecordComplexF32(vsa,VI_NULL,0,-1,timeout,capturedIQ, &captureInfo,Tag,Key);

		if(status == 0 && captureInfo.actualSamples == NumSamples)
		{
			// Fetch of IQ samples was successful			
			for(int i = 0; i < NumSamples; i++)
			{
				iBuffer[i] = capturedIQ[i].real;
				qBuffer[i] = capturedIQ[i].imaginary;
			}

#if 0
			// Helper method to store I and Q to a file.
			DumpIQ("WLAN_IQ_DUMP.csv", iBuffer,qBuffer,NumSamples,VstLevelCorrection);
#endif
		}
		else
		{
			std::cout << "An error occured during capture ( the number of samples returned does not match what was asked for )!" << std::endl;
		}

		// Free the allocated memory.
		delete capturedIQ;
	}
	else
	{
		std::cout << "An error occured during capture!" << std::endl;
	}
}


//****************************************************************************** 
// Function          : ErrorCheckSpectrum
// Globals Used      : None
// Globals Modified  : None
// Description       : Using a Cobham Spectrum Analysis instance and a status code the status
// code is checked to see if it is either no error, an error or a warning occured.
//******************************************************************************
void ErrorCheckSpectrum(afSpectrumDllObjectID SpectrumObj, long Status)
{	
	char errMsg[256];

	if ( Status > 0 ) // Warning
	{
		// Retrieve the error message based on the status code.
		afSpectrumDll_GetErrorMsg(SpectrumObj, Status, errMsg, sizeof(errMsg));

		std::cout << "Warning : ";

		// Loop through the array of chars printing them out to the STDOUT.
		for( int i = 0 ; i < strlen(errMsg) ; i ++ )
		{
			std::cout << errMsg[i]; 
		}
		std::cout << std::endl;
	}
	else if ( Status < 0 ) // Error
	{
		// Retrieve the error message based on the status code.
		afSpectrumDll_GetErrorMsg(SpectrumObj, Status, errMsg, sizeof(errMsg));

		std::cout << "Error : ";			

		// Loop through the array of chars printing them out to the STDOUT.
		for( int i = 0 ; i < strlen(errMsg) ; i ++ )
		{
			std::cout << errMsg[i]; 
		}
		std::cout << std::endl;		
	}			

//	delete errMsg;
}



//****************************************************************************** 
// Function          : GetStitchingBandwidth
// Globals Used      : None
// Globals Modified  : None
// Description       : Returns the correct stitching bandwidth to use based
// on the measurement span, centre frequency and VST model.
//******************************************************************************
double GetStitchingBandwidth(double Centrefrequency, double MeasurementSpan, model_t Model)
{
	double startFreq = Centrefrequency - (MeasurementSpan /2);
	double stopFreq = Centrefrequency + (MeasurementSpan /2);

	if ( Model == VST_5644R || Model == VST_5645R )
	{
		if ( startFreq  >= MinimumFrequencyRegion1 && startFreq <= MaximumFrequencyRegion1) // Starts in Region 1.
		{
			return MaximumBandwidthRegion1_5644R_5645R;
		}
		else
		{
			return MaximumBandwidthRegion2_5644R_5645R;
		}					
	}
	else if ( Model == VST_5646R )
	{
		if ( startFreq  >= MinimumFrequencyRegion1 && startFreq <= MaximumFrequencyRegion1) // Starts in Region 1.
		{
			return MaximumBandwidthRegion1_5646R;
		}
		else if ( startFreq  >= MinimumFrequencyRegion2_5646R && startFreq <= MaximumFrequencyRegion2_5646R) // Starts in Region 2.
		{
			return MaximumBandwidthRegion2_5646R;
		}
		else // Starts in Region 3.
		{
			return MaximumBandwidthRegion3_5646R;
		}
	}
}





//****************************************************************************** 
// Function          : SpectralAnalysis
// Globals Used      : None
// Globals Modified  : None
// Description       : Creates a Specrum Analysis object and does a sticthed spectrum
// measurments.
void SpectralAnalysis(ViSession vsa, ViReal64 CarrierFrequency, ViReal64 ReferenceLevel, ViReal64 DwellTime, double MeasurementSpan, model_t Model)
{
	long status;	
	long fftSize = 0;
	double digitizerStitchingBw = GetStitchingBandwidth(CarrierFrequency,MeasurementSpan, Model);
	std::cout << "Stitching Bandwidth : " << digitizerStitchingBw << " Hz" <<std::endl;
	double sampleRate = digitizerStitchingBw * 2;
	int numberOfSamples = (int)(DwellTime * sampleRate);
	float *iData = new float[numberOfSamples];
	float *qData = new float[numberOfSamples];
	double key;
	double tag;

	//
	// Configure Analysis Library
	//	

	// Define the measurements to make.
	AFSPECTRUM_MEASUREMENT measurements = afSpectrumDllMeasPowerVsFreq;

	// Define the traces we'll read back.
	AFSPECTRUM_TRACE_DATA traces		= afSpectrumDllTracePowerVsFreq;

	// Create Spectrum Analysis object.
	afSpectrumDllObjectID spectrumObj;
	status = afSpectrumDll_CreateObject(&spectrumObj);
	ErrorCheckSpectrum(spectrumObj,status);

	// Set level correction to account for VST IQ W -> mW and VST impedance.
	status = afSpectrumDll_RfLevelCal_Set(spectrumObj, VstLevelCorrection);
	ErrorCheckSpectrum(spectrumObj,status);			

	// Set the measurement span. This is the entire spectrum that is to be measured. ( this can be > digitizer bandwidth )
	status = afSpectrumDll_MeasurementSpan_Set(spectrumObj,MeasurementSpan);
	ErrorCheckSpectrum(spectrumObj,status);

	// Set the digitizers actual maximum bandwidth. For the VST to avoid LO and aliasing issues we set it to less than the maximum to avoid these issues.
	status = afSpectrumDll_DigitizerSpan_Set(spectrumObj, digitizerStitchingBw);
	ErrorCheckSpectrum(spectrumObj,status);

	// Set the sample rate. Make sure this is equal to the sample rate that will be used to capture the data.
	status = afSpectrumDll_SamplingFreq_Set(spectrumObj,sampleRate);
	ErrorCheckSpectrum(spectrumObj,status);

	status = afSpectrumDll_WindowType_Set(spectrumObj,afSpectrumWindow_GaussianNoise);
	ErrorCheckSpectrum(spectrumObj,status);

	status = afSpectrumDll_RecommendedFFTSize_Get(spectrumObj, &fftSize);
	ErrorCheckSpectrum(spectrumObj,status);

	status = afSpectrumDll_FFTSize_Set(spectrumObj,fftSize);
	ErrorCheckSpectrum(spectrumObj,status);

	// Set the X-Axis to the correct frequency.
	status = afSpectrumDll_Xaxis_CentreFreq_Set(spectrumObj,CarrierFrequency);
	ErrorCheckSpectrum(spectrumObj,status);

	status = afSpectrumDll_AnalysisMode_Set(spectrumObj, afSpectrumDllAnalysisModeAllIQ);
	ErrorCheckSpectrum(spectrumObj,status);

	long numberOfStitches = 0;

	// Work out how many stitches will be required.
	status = afSpectrumDll_Spectrum_NumStitches_Get(spectrumObj,&numberOfStitches);
	ErrorCheckSpectrum(spectrumObj,status);

	std::cout << "Number of Stitches Required : " << numberOfStitches <<std::endl;

	if ( numberOfStitches > 1)
	{                              
		for (int stitchIndex = 0; stitchIndex < numberOfStitches; stitchIndex++)
		{
			/* Set the stitch index .... */
			status = afSpectrumDll_Spectrum_StitchIndex_Set(spectrumObj,stitchIndex+1);
			ErrorCheckSpectrum(spectrumObj,status);

			/* Set the Digitizer to the correct centre frequency for this stitch */
			double offsetFreqForStitch = 0;
			status = afSpectrumDll_Spectrum_StitchOffsetFreq_Get(spectrumObj,&offsetFreqForStitch);
			ErrorCheckSpectrum(spectrumObj,status);						

			// Perform the capture.
			Capture(vsa, CarrierFrequency + offsetFreqForStitch, ReferenceLevel, numberOfSamples, sampleRate,LO_AUTO,digitizerStitchingBw, Model, iData, qData,&key,&tag);

			// Call the analyse method passing in the measurements to make and the IQ.
			status = afSpectrumDll_Analyse(spectrumObj, measurements, iData, qData,numberOfSamples);
		}
	}
	long traceLength = 0;

	// Work out how many points in the trace we're going to pull back.
	status = afSpectrumDll_GetTraceDataLength(spectrumObj,traces,&traceLength);
	ErrorCheckSpectrum(spectrumObj,status);

	// Create an array to hold them
	double *xTraceData = new double[traceLength];
	double *yTraceData = new double[traceLength];

	// Retreive the trace
	status = afSpectrumDll_GetTraceData(spectrumObj,traces,xTraceData,yTraceData,traceLength);
	ErrorCheckSpectrum(spectrumObj,status);

	// Write the X vs Y Spectral trace data to file.
	DumpXY("Stitched Spectrum.csv",xTraceData,yTraceData,traceLength);

	// Free Allocated Memory

	delete[] iData;
	delete[] qData;
	delete[] xTraceData;
	delete[] yTraceData;

	// Destroy the WLAN object.
	CloseSpectrum(spectrumObj);
}


//****************************************************************************** 
// Function          : _tmain
// Globals Used      : None
// Globals Modified  : None
// Description       : Console App entry point.
//******************************************************************************
int _tmain(int argc, _TCHAR* argv[])
{
	ViSession vsg = VI_NULL;
	ViSession vsa = VI_NULL;
	std::vector<ViReal64> I;
	std::vector<ViReal64> Q;
	ViReal64 sampleRate = 80e6; // 80 MHz.
	ViReal64 dwellTime = 1e-3; // 1 ms.
	ViReal64 level		= -20;	// -20 dBm
	ViReal64 referenceLevel = -20; // 20 dBm
	ViReal64 headRoom = 16; // 16 dB of headroom.
	int myChoice = 0;
	
	std::cout << "VST Example 6 - Spectral Stitching With the VST" << std::endl;
	std::cout << "-----------------------------------------------" << std::endl;
	std::cout << std::endl;
	// Boot and configure the RFSG.
	std::cout << "Booting and configuring RFSG." << std::endl;
	vsg = BootRFSG();
	std::cout << "Booting and configuring RFSA." << std::endl;
	vsa = BootRFSA();
	std::cout << std::endl;
	std::cout << "********** Main Menu **********" << std::endl;
	std::cout << "(1): Perform a Stitched Specral Measurement of 80 MHz using a VST 5644R." << std::endl;
	std::cout << "(2): Perform a Stitched Specral Measurement of 80 MHz using a VST 5644R close to 6GHz." << std::endl;
	std::cout << "(3): Perform a Stitched Specral Measurement of 80 MHz using a VST 5644R with the capture over two regions." << std::endl;
	std::cout << "(5): Exit" << std::endl;
	
	do
	{
		std::cin >> myChoice;

		switch(myChoice)
		{
		case 1:					
			PlayAiqWaveform(vsg,"WLAN_11AC_40MHz_MCS7.aiq",1e9,level);
			SpectralAnalysis(vsa,1e9,referenceLevel + headRoom,dwellTime,80e6, VST_5644R);			
			break;			
		case 2:
			PlayAiqWaveform(vsg,"WLAN_11AC_40MHz_MCS7.aiq",5.95e9,level);
			SpectralAnalysis(vsa,5.95e9,referenceLevel + headRoom,dwellTime,80e6, VST_5644R);			
			break;
		case 3:
			PlayAiqWaveform(vsg,"WLAN_11AC_40MHz_MCS7.aiq",110e6,level);
			SpectralAnalysis(vsa,110e6,referenceLevel + headRoom,dwellTime,80e6, VST_5644R);			
			break;
		default:
			std::cout << "ERROR! You have selected an invalid choice." << std::endl;
			break;
		}	
		std::cout << "********** Main Menu **********" << std::endl;
		std::cout << "(1): Perform a Stitched Specral Measurement of 80 MHz in Region 2." << std::endl;
		std::cout << "(2): Perform a Stitched Specral Measurement of 80 MHz using a VST 5644R close to 6GHz." << std::endl;
		std::cout << "(3): Perform a Stitched Specral Measurement of 80 MHz using a VST 5644R with the capture over two regions." << std::endl;
		std::cout << "(5): Exit" << std::endl;
	}while(myChoice !=5);
	
	// Cleanup

	I.clear();
	Q.clear();

	CloseRFSG(vsg);
	CloseRFSA(vsa);
	

	return 0;
}

