//****************************************************************************** //
// $Id: $
// 
// Original Author: <name> // Creation Date: <date>
// $Author: $
// $Date: $
// $Revision: $
//
//*****************************************************************************
//
// Description: Example code showing how to generate a CW tone or playback a waveform from the VST using RFSG.
//
// Copyright (c) 2005-20015, Aeroflex International Ltd.
//
//
//*****************************************************************************

#include "stdafx.h"
#include "niRFSG.h"
#include "niRFSA.h"
#include "afVstWrapper.h"
#include <stdio.h>
#include <vector>
#include <fstream>
#include <string>
#include <iostream>
#include <direct.h>
#include <math.h>
#include "afWlanAnalysis.h"

// The real and imaginary parts of this complex data array correspond to the in-phase (I) and quadrature-phase (Q) data,
// respectively. To calculate the instantaneous power of a sampled I/Q point, use the equation (I^2 + Q^2)/2R, where R is
// the input impedance in ohms (O). For NI RF vector signal analyzers, R = 50 O. 
// For use with analysis library a fixed level correction defined below can account for the W -> nW conversion of the IQ.
// 10 * log10(1000.0 /* W -> mW */ / (2 /* See NI help file */ * 50.0 /* input impedance - see NI help file */));
static const double VstLevelCorrection = 10 * log10(1000.0 / (2 * 50.0 ));

//****************************************************************************** 
// Function          : DumpIQ
// Globals Used      : None
// Globals Modified  : None
// Description       : Writes out the IBuffer, QBuffer and power for the IQ pair.
//******************************************************************************
void DumpIQ(char* fileName, float* iBuffer, float* qBuffer, int NumberOfSamples, float LevelCorrection)
{
	  std::ofstream myfile;
	  float power;
	  float powerIndBm;

      myfile.open (fileName);     
      myfile << "I,Q,Power(dBm),\n";
      
	  for( int index = 0; index < NumberOfSamples; index++)
	  {
		  power = iBuffer[index] * iBuffer[index] + qBuffer[index] * qBuffer[index];
		  powerIndBm = 10.0 * log10f(power) + LevelCorrection;
		  myfile << iBuffer[index] << "," << qBuffer[index] << "," << powerIndBm << "," << power << "\n";
	  }
	  
      myfile.close();     
}

//****************************************************************************** 
// Function          : ErrorCheckRFSA
// Globals Used      : None
// Globals Modified  : None
// Description       : Using a RFSA VISA session and a status code the status
// code is checked to see if it is either no error, an error or a warning.
//******************************************************************************
void ErrorCheckRFSA(ViSession Vsa, ViStatus Status)
{
	ViChar errMsg[256];

	if ( Status > 0 ) // Warning
	{		
		// Retrieve the error message based on the status code.
		niRFSA_error_message(Vsa, Status, errMsg);

		std::cout << "Warning : ";

		// Loop through the array of chars printing them out to the STDOUT.
		for( int i = 0 ; i < strlen(errMsg) ; i ++ )
		{
			std::cout << errMsg[i]; 
		}
		std::cout << std::endl;		
	}
	else if ( Status < 0 ) // Error
	{
		ViChar errMsg[256];

		// Retrieve the error message based on the status code.
		niRFSA_error_message(Vsa, Status, errMsg);

		std::cout << "Error : ";			

		// Loop through the array of chars printing them out to the STDOUT.
		for( int i = 0 ; i < strlen(errMsg) ; i ++ )
		{
			std::cout << errMsg[i]; 
		}
		std::cout << std::endl;		
	}	

	//delete errMsg;
}

//****************************************************************************** 
// Function          : ErrorCheckRFSG
// Globals Used      : None
// Globals Modified  : None
// Description       : Using a RFSG VISA session and a status code the status
// code is checked to see if it is either no error, an error or a warning.
//******************************************************************************
void ErrorCheckRFSG(ViSession Vsg, ViStatus Status)
{
	ViChar errMsg[256];

	if ( Status > 0 ) // Warning
	{
		// Retrieve the error message based on the status code.
		niRFSG_error_message(Vsg, Status, errMsg);

		std::cout << "Warning : ";

		// Loop through the array of chars printing them out to the STDOUT.
		for( int i = 0 ; i < strlen(errMsg) ; i ++ )
		{
			std::cout << errMsg[i]; 
		}
		std::cout << std::endl;		
	}
	else if ( Status < 0 ) // Error
	{		
		// Retrieve the error message based on the status code.
		niRFSG_error_message(Vsg, Status, errMsg);

		std::cout << "Error : ";			

		// Loop through the array of chars printing them out to the STDOUT.
		for( int i = 0 ; i < strlen(errMsg) ; i ++ )
		{
			std::cout << errMsg[i]; 
		}
		std::cout << std::endl;		
	}	

	//delete errMsg;
}

//****************************************************************************** 
// Function          : ErrorCheckWrapper
// Globals Used      : None
// Globals Modified  : None
// Description       : Using a status code generated from API calls to the afVSTWrapper
// the code is checked to see if it is either no error, an error or a warning occured.
//******************************************************************************
void ErrorCheckWrapper(long Status)
{
	std::string errorMessage;

	if ( Status < 0 ) // Error
	{		
		switch(Status)
		{
		case afVstWrapper_eUnknownError:
			{
				errorMessage = "Unknown error";
				break;
			}
		case afVstWrapper_eFailedToLoadRFSGLibrary:
			{
				errorMessage = "The RFSG LIbrary could not be loaded";
				break;
			}
		case afVstWrapper_eFailedToLoadRFSALibrary:
			{
				errorMessage = "The RFSA LIbrary could not be loaded";
				break;
			}
		case afVstWrapper_eFailedToQuerySerialNumber:
			{
				errorMessage = "An error occurred while attempting to read the VST serial number";
				break;
			}
		case afVstWrapper_eFailedToConnectToLicenseServer:
			{
				errorMessage = "A connection to the license service could not be created";
				break;
			}
		case afVstWrapper_eFailedToQueryLicenseServer:
			{
				errorMessage = "An error occurred while querying the license service for password details";
				break;
			}
		case afVstWrapper_eFailedToSignIQ:
			{
				errorMessage = "An error occurred while creating the key/tag for the IQ data";
				break;
			}
		case afVstWrapper_eNoPasswordForOption:
			{
				errorMessage = "No password is available for the specified option";
				break;
			}
		case afVstWrapper_eRequiredOptionNotEnabled:
			{
				errorMessage = "An option required to load the AIQ file is not enabled";
				break;
			}
		case afVstWrapper_eAiqFileCreatedWithWrongIQCreatorVersion:
			{
				errorMessage = "The AIQ file is using an obsolete format";
				break;
			}
		case afVstWrapper_eAiqFileOpenError:
			{
				errorMessage = "An error occurred while opening the AIQ file";
				break;
			}
		case afVstWrapper_eAiqFileReadError:
			{
				errorMessage = "An error occurred while reading the AIQ file";
				break;
			}
		case afVstWrapper_eAiqFileFormatError:
			{
				errorMessage = "The AIQ file contains data in an invalid format";
				break;
			}
		case afVstWrapper_eAiqFileChecksumError:
			{
				errorMessage = "The AIQ file checksum does not match the file contents";
				break;
			}
		case afVstWrapper_eAiqFileLoadFailure:
			{
				errorMessage = "An unexpected error occurred while loading the AIQ file";
				break;
			}
		case afVstWrapper_eTooManyIQSamples:
			{
				errorMessage = "Too many IQ samples";
				break;
			}
		case afVstWrapper_eMemAllocFailure:
			{
				errorMessage = "Memory allocation error";
				break;
			}
		case afVstWrapper_eMarkerBufferTooSmall:
			{
				errorMessage = "The supplied marker buffer is too small";
				break;
			}
		case afVstWrapper_eFunctionNotImplemented:
			{
				errorMessage = "A required function is not implemented in the RFSA or RFSG driver - check correct version installed.";
				break;
			}
		default:
			{
				errorMessage = "Unknown Error Code";
			}
		}

		std::cout << "Error : " << errorMessage.c_str() << std::endl;
	}		
}


//****************************************************************************** 
// Function          : BootRFSA
// Globals Used      : None
// Globals Modified  : None
// Description       : Boots the RFSA within the VST and returns a reference to
// the RFSA.
//******************************************************************************
unsigned long BootRFSA()
{
	ViStatus status;
	ViSession vsa = VI_NULL;
	ViReal64 referenceClockRate = 10e6; // 10 MHz ( VST only supports 10 MHz )

	// Initiate VSA session		
	status = niRFSA_init("RIO0", VI_TRUE, VI_TRUE, &vsa);					
	ErrorCheckRFSA(vsa,status);

	// Configure 10MHz reference
	status = niRFSA_ConfigureRefClock(vsa, NIRFSA_VAL_ONBOARD_CLOCK_STR, referenceClockRate);
	ErrorCheckRFSA(vsa,status);

	// Commit the settings to make clock settings active
	status = niRFSA_Commit(vsa);
	ErrorCheckRFSA(vsa,status);

	std::cout << "VST - RFSA Booted and Configured for 10 MHz reference clock. : " << std::endl;			

	return vsa;
}

//****************************************************************************** 
// Function          : BootRFSG
// Globals Used      : None
// Globals Modified  : None
// Description       : Boots the RFSG within the VST and returns a reference to
// the RFSG.
//******************************************************************************
unsigned long BootRFSG()
{
	ViStatus status;
	ViSession vsg = VI_NULL;
	ViReal64 referenceClockRate = 10e6; // 10 MHz ( VST only supports 10 MHz )

	// Initiate VSG session
	status = niRFSG_init("RIO0", VI_TRUE, VI_TRUE, &vsg);
	ErrorCheckRFSG(vsg,status);

	// Configure 10MHz reference
	status = niRFSG_ConfigureRefClock(vsg, NIRFSG_VAL_ONBOARD_CLOCK_STR, referenceClockRate); // onboard reference
	ErrorCheckRFSG(vsg,status);

	// Export reference clock to REF out connector (if required)
	status = niRFSG_ExportSignal(vsg, NIRFSG_VAL_REF_CLOCK, "", NIRFSG_VAL_REF_OUT_STR);
	ErrorCheckRFSG(vsg,status);

	// Commit the settings to make clock settings active
	status = niRFSG_Commit(vsg);
	ErrorCheckRFSG(vsg,status);

	std::cout << "VST - RFSG Booted and Configured for 10 MHz reference clock. : " << std::endl;		

	return vsg;
}

//****************************************************************************** 
// Function          : BootRFSG
// Globals Used      : None
// Globals Modified  : None
// Description       : Boots the RFSG within the VST and returns a reference to
// the RFSG.
//******************************************************************************
void CloseRFSA(ViSession vsa)
{
	ViStatus status;

	// Close session
	status = niRFSA_close(vsa);
	ErrorCheckRFSA(vsa,status);
}

//****************************************************************************** 
// Function          : BootRFSG
// Globals Used      : None
// Globals Modified  : None
// Description       : Boots the RFSG within the VST and returns a reference to
// the RFSG.
//******************************************************************************
void CloseRFSG(ViSession vsg)
{
	ViStatus status;

	// Close session
	status = niRFSG_close(vsg);
	ErrorCheckRFSG(vsg,status);
}

//****************************************************************************** 
// Function          : CloseWLAN
// Globals Used      : None
// Globals Modified  : None
// Description       : 
//******************************************************************************
void CloseWLAN(afWlanObjectID wlanObj)
{
	std::cout << "Destroying WLAN Analysis Object." << std::endl;
	long status = afWlanDll_DestroyObject(wlanObj);
	
	if (status != 0 )
	{
		std::cout << "An error occured shutting the WLAN Analysis Library Down" << std::endl;
	}
}

//****************************************************************************** 
// Function          : BootRFSG
// Globals Used      : None
// Globals Modified  : None
// Description       : Generates a CW tone at a given frequency and level.
//******************************************************************************
void PlayAiqWaveform(ViSession vsg, ViConstString fileNameAiq, ViReal64 frequency, ViReal64 level)
{
	ViStatus status;
	ViBoolean done;
	ViReal64 headRoom = 0;
	ViReal64 samplingRate;
	ViReal64 crestFactor;
	ViReal64 peakPowerAdjust;
	ViReal64 averagePowerAdjust;
	ViInt32 markerBufferLength = 256;
	ViInt32 fileMarkerCount;
	afRFSG_MarkerDetails_t *markerBuffer = new afRFSG_MarkerDetails_t[markerBufferLength];
	char* drive = new char[256];
	char* folder = new char[256];
	char* filenameNoextension = new char[256];
	char* extension = new char[16];
		
	// Aborts generation & put VSG in configuration mode
	status = niRFSG_Abort(vsg);
	ErrorCheckRFSA(vsg,status);

	//
	// Configure all settings that require the VSG to be in configuration mode
	//

	// RMS mode will automatically rescale ARB files to maximise DAC dynamic range.
	// PEAK mode will treat the RF level as being the peak value, so the RMS value will usually be lower.  ARB files are not re-scaled.
	status = niRFSG_ConfigurePowerLevelType(vsg, NIRFSG_VAL_AVERAGE_POWER);
	ErrorCheckRFSA(vsg,status);

	// Clear All wavefroms so we don't get an error if the waveform is already loaded.
	status = niRFSG_ClearAllArbWaveforms (vsg);
	ErrorCheckRFSA(vsg,status);	

	// Remove the path and extension from the file so that a name can be associated with the IQ on the VST.
	_splitpath(fileNameAiq,drive,folder,filenameNoextension,extension);
	
	// Load the .aiq file using the wrapper and set the sample rate and on board filename.
	status = afVstWrapper_RFSG_LoadAiqFile(vsg, VI_FALSE ,fileNameAiq,filenameNoextension,headRoom,&samplingRate,&crestFactor, &peakPowerAdjust, &averagePowerAdjust,markerBuffer,markerBufferLength,&fileMarkerCount);	
	ErrorCheckWrapper(status);

	// Display Marker Positions
	for(int index = 0; index < fileMarkerCount; index++)
	{ 
		std::cout << "Marker : " << markerBuffer[index].markerNumber << "\tOn Sample : " << markerBuffer[index].onSample << "\tOff Sample : " << markerBuffer[index].offSample << std::endl; 		
	}

	std::cout << "" << std::endl;
	std::cout << "" << std::endl;
	std::cout << "" << std::endl;

	// Now Auto-Generate Marker Script
	std::cout << "script wlan11ACExample4" << std::endl;
	std::cout << "" << std::endl;
	std::cout << "generate " << filenameNoextension;	

	// Set ARB playback sample rate
	status = niRFSG_SetAttributeViReal64(vsg, VI_NULL, NIRFSG_ATTR_IQ_RATE, samplingRate);
	ErrorCheckRFSA(vsg,status);

	// Set generation mode to ARB Waveform.
	status = niRFSG_ConfigureGenerationMode(vsg, NIRFSG_VAL_ARB_WAVEFORM);
	ErrorCheckRFSA(vsg,status);

	// Configure the ARB to playback continuously or a finite numberof times.
	status = niRFSG_SetAttributeViBoolean(vsg, VI_NULL, NIRFSG_ATTR_ARB_WAVEFORM_REPEAT_COUNT_IS_FINITE, VI_FALSE); // VI_FALSE for continuous playback VI_TRUE to play back n-times.
	ErrorCheckRFSA(vsg,status);

	// Configure number of times ARB file is played
	status = niRFSG_SetAttributeViInt32(vsg, VI_NULL, NIRFSG_ATTR_ARB_WAVEFORM_REPEAT_COUNT, 1000); 
	ErrorCheckRFSA(vsg,status);

	// Disable start trigger, so generation will commence immediately after _Initiate() called
	status = niRFSG_DisableStartTrigger(vsg);
	ErrorCheckRFSA(vsg,status);

	// Configure frequency & level using higher level function
	status = niRFSG_ConfigureRF(vsg, frequency, level + averagePowerAdjust);
	ErrorCheckRFSA(vsg,status);

	// Enable output
	status = niRFSG_ConfigureOutputEnabled(vsg, VI_TRUE);
	ErrorCheckRFSA(vsg,status);

	// Enable software start trigger
	status = niRFSG_ConfigureSoftwareStartTrigger(vsg);
	ErrorCheckRFSA(vsg,status);

	// If more than one waveform is loaded you require to select the correct file.
	status = niRFSG_SelectArbWaveform(vsg,filenameNoextension);
	ErrorCheckRFSA(vsg,status);

	// Check what waveform we're actually using:
	ViChar* selectedWaveform = new ViChar[256];

	status = niRFSG_GetAttributeViString(vsg,VI_NULL, NIRFSG_ATTR_ARB_SELECTED_WAVEFORM,256,selectedWaveform);
    ErrorCheckRFSA(vsg,status);

	// Initiate generation (VSG now in generation mode)
	status = niRFSG_Initiate(vsg);
	ErrorCheckRFSA(vsg,status);

	// Check Generation Status. ( returns 0 (generation active) )
	status = niRFSG_CheckGenerationStatus(vsg, &done); 
	ErrorCheckRFSA(vsg,status);

	// VSG now in generation mode but awaiting a software trigger to commence ARB playback
	status = niRFSG_SendSoftwareEdgeTrigger(vsg, NIRFSG_VAL_START_TRIGGER, ""); // send software trigger
	ErrorCheckRFSA(vsg,status);

	// Deallocate allocated memory.
	delete(markerBuffer);
	delete(drive);
	delete(folder);
	delete(filenameNoextension);
	delete(extension);
}

//****************************************************************************** 
// Function          : Capture
// Globals Used      : None
// Globals Modified  : None
// Description       : Captures IQ for a specified dwell time at a specified frequency and input level at a specified sample rate.
//******************************************************************************
void Capture(ViSession vsa, ViReal64 CarrierFrequency, ViReal64 ReferenceLevel, int NumSamples, ViReal64 SampleRate, float* iBuffer, float* qBuffer, double* Key, double* Tag )
{
	ViStatus status;

	//
	// Configure VST for capture
	//
	status = niRFSA_ConfigureAcquisitionType(vsa, NIRFSA_VAL_IQ); // IQ capture mode
	ErrorCheckRFSA(vsa,status);

	status = niRFSA_ConfigureIQCarrierFrequency(vsa, VI_NULL, CarrierFrequency); // Frequency
	ErrorCheckRFSA(vsa,status);

	status = niRFSA_ConfigureReferenceLevel(vsa, VI_NULL, ReferenceLevel); // Reference level
	ErrorCheckRFSA(vsa,status);

	// Set sample rate to that required by the analysis library for the selected measurements
	status = niRFSA_ConfigureIQRate(vsa, VI_NULL, SampleRate);
	ErrorCheckRFSA(vsa,status);
	
	status = niRFSA_ConfigureNumberOfSamples(vsa, VI_NULL, VI_TRUE, NumSamples); // Number of samples
	ErrorCheckRFSA(vsa,status);

	status = niRFSA_ConfigureNumberOfRecords(vsa, VI_NULL, VI_TRUE, 1); // Number of records
	ErrorCheckRFSA(vsa,status);

	status = niRFSA_DisableStartTrigger(vsa); // immediate trigger
	ErrorCheckRFSA(vsa,status);

	status = niRFSA_Initiate(vsa); // start capture
	ErrorCheckRFSA(vsa,status);

	if(status == 0)
	{
		// Fetch IQ data
		NIComplexNumberF32 *capturedIQ = new NIComplexNumberF32[NumSamples];

		status = niRFSA_SetAttributeViInt32(vsa, VI_NULL, NIRFSA_ATTR_FETCH_RELATIVE_TO, NIRFSA_VAL_FIRST_SAMPLE);
		ErrorCheckRFSA(vsa,status);

		niRFSA_wfmInfo captureInfo;
		ViReal64 timeout = 1.0;
		
		// Use AfVstWrapper to fetch IQ samples instead of RFSA niRFSA_FetchIQSingleRecordComplexF32(...) method call so that Tag and Key
		// are generated ( if license is fitted ) so that the signal can be analysed with the Cobham WLAN analysis library.
		status = afVstWrapper_RFSA_FetchIQSingleRecordComplexF32(vsa,VI_NULL,0,-1,timeout,capturedIQ, &captureInfo,Tag,Key);

		if(status == 0 && captureInfo.actualSamples == NumSamples)
		{
			// Fetch of IQ samples was successful			
			for(int i = 0; i < NumSamples; i++)
			{
				iBuffer[i] = capturedIQ[i].real;
				qBuffer[i] = capturedIQ[i].imaginary;
			}

#if 0
			// Helper method to store I and Q to a file.
			DumpIQ("WLAN_IQ_DUMP.csv", iBuffer,qBuffer,NumSamples,VstLevelCorrection);
#endif
		}
		else
		{
			std::cout << "An error occured during capture ( the number of samples returned does not match what was asked for )!" << std::endl;
		}

		// Free the allocated memory.
		delete capturedIQ;
	}
	else
	{
		std::cout << "An error occured during capture!" << std::endl;
	}
}


//****************************************************************************** 
// Function          : ErrorCheckWLAN
// Globals Used      : None
// Globals Modified  : None
// Description       : Using a Cobham WLAN Analysis instance and a status code the status
// code is checked to see if it is either no error, an error or a warning occured.
//******************************************************************************
void ErrorCheckWLAN(afWlanObjectID WlanObj, long Status)
{	
	char errMsg[256];

	if ( Status > 0 ) // Warning
	{
		// Retrieve the error message based on the status code.
		afWlanDll_GetErrorMsg(WlanObj, Status, errMsg, sizeof(errMsg));

		std::cout << "Warning : ";

		// Loop through the array of chars printing them out to the STDOUT.
		for( int i = 0 ; i < strlen(errMsg) ; i ++ )
		{
			std::cout << errMsg[i]; 
		}
		std::cout << std::endl;
	}
	else if ( Status < 0 ) // Error
	{
		// Retrieve the error message based on the status code.
		afWlanDll_GetErrorMsg(WlanObj, Status, errMsg, sizeof(errMsg));

		std::cout << "Error : ";			

		// Loop through the array of chars printing them out to the STDOUT.
		for( int i = 0 ; i < strlen(errMsg) ; i ++ )
		{
			std::cout << errMsg[i]; 
		}
		std::cout << std::endl;		
	}			

//	delete errMsg;
}


//****************************************************************************** 
// Function          : WlanAnalysis
// Globals Used      : None
// Globals Modified  : None
// Description       : Creates a WLAN Analysis object and tries to perform the Power,
// OBW and Modulation accuracy measurments on a 802.11AC VHT 40MHz 
void WlanAnalysis(ViSession vsa, int NumSamples, ViReal64 SampleRate, float* iBuffer, float* qBuffer, double Key, double Tag )
{
	long status;
	
	//
	// Configure Analysis Library
	//	
	AFWLAN_MEASUREMENT measurements = afWlanLocateBurst | afWlanBurstPower | afWlanMeasOccupiedBandwidth | afWlanModAccuracy;

	afWlanObjectID wlanObj;
	status = afWlanDll_CreateObject(&wlanObj);
	ErrorCheckWLAN(wlanObj,status);

	// Set level correction to account for VST IQ W -> mW and VST impedance.
	status = afWlanDll_RfLevelCal_Set(wlanObj, VstLevelCorrection);
	ErrorCheckWLAN(wlanObj,status);

	// Set for 802.11AC VHT signal.
	status = afWlanDll_AnalysisMode_Set(wlanObj, afWlanAnalysisMode11acVHT);
	ErrorCheckWLAN(wlanObj,status);

	// Auto detect signal fields.
	status = afWlanDll_AutoDetectSignalFields_Set(wlanObj, AF_TRUE);
	ErrorCheckWLAN(wlanObj,status);

	// Define signal bandwidth to be 40MHz.
	status = afWlanDll_ChannelBandwidth_Set(wlanObj, afWlanChannelBandwidth40MHz);
	ErrorCheckWLAN(wlanObj,status);

	status = afWlanDll_AntennaNumber_Set(wlanObj, 0);
	ErrorCheckWLAN(wlanObj,status);

	status = afWlanDll_BurstProfile_Mode_Set(wlanObj, afWlanAveragePower);
	ErrorCheckWLAN(wlanObj,status);

	// Set amplitude tracking to off.
	status = afWlanDll_PilotTracking_Amplitude_Set(wlanObj, AF_FALSE);
	ErrorCheckWLAN(wlanObj,status);

	// Set phase tracking to on.
	status = afWlanDll_PilotTracking_Phase_Set(wlanObj, AF_TRUE);
	ErrorCheckWLAN(wlanObj,status);

	// Set timing tracking to off.
	status = afWlanDll_PilotTracking_Timing_Set(wlanObj, AF_FALSE);
	ErrorCheckWLAN(wlanObj,status);

	// Define the OFDM equalization setting to be over the preamble only.
	status = afWlanDll_Ofdm_EqualizationMode_Set(wlanObj, afWlanOfdmEqPreamble);
	ErrorCheckWLAN(wlanObj,status);

	// Define the analysis library to auto-detect the number of symbols
	status = afWlanDll_NumSymbolsToAnalyse_AutoDetect_Set(wlanObj, AF_TRUE);
	ErrorCheckWLAN(wlanObj,status);

	status = afWlanDll_ChannelOffset_Set(wlanObj, 0);
	ErrorCheckWLAN(wlanObj,status);

	status = afWlanDll_CaptureOffset_Set(wlanObj, 0);
	ErrorCheckWLAN(wlanObj,status);

	// Spectrum parameters
	status = afWlanDll_SpectralMask_Type_Set(wlanObj, afWlanSpectralMask11ac);
	ErrorCheckWLAN(wlanObj,status);

	status = afWlanDll_Spectrum_AnalysisMode_Set(wlanObj, afWlanSpectrumAnalysisNonGated);
	ErrorCheckWLAN(wlanObj,status);

	status = afWlanDll_Spectrum_FreqAxis_Centre_Set(wlanObj, 1e9);
	ErrorCheckWLAN(wlanObj,status);

	// If performing any spectrum measurements, the analysis library needs to know the bandwidth of the
	// VSA to determine whether stitching is required.
	ViReal64 instBW;
	status = niRFSA_GetAttributeViReal64(vsa, VI_NULL, NIRFSA_ATTR_DEVICE_INSTANTANEOUS_BANDWIDTH, &instBW); // query VSA bandwidth
	ErrorCheckRFSA(vsa,status);

	status = afWlanDll_DigitizerSpan_Set(wlanObj, instBW);
	ErrorCheckWLAN(wlanObj,status);
	// Query the minimum measurement span required for the desired measurements
	//
	// Note that all analysis library parameter that impact the measurement span must be set before querying, namely:
	// Analysis Mode, Channel BW, Channel Offset, Spectral Mask Type
	double minMeasSpan;
	status = afWlanDll_GetMinMeasurementSpan(wlanObj, measurements, &minMeasSpan);
	ErrorCheckWLAN(wlanObj,status);

	if(minMeasSpan <= instBW)
	{
		// Minimum measurement span is less than the VSA bandwidth - no stitching required
		status = afWlanDll_MeasurementSpan_Set(wlanObj, minMeasSpan);
		ErrorCheckWLAN(wlanObj,status);
	}
	else
	{
		// Minimum measurement span exceeds the VSA bandwidth, so stitching will be required
		std::cout << "Signal bandwith exceeds HW bandwidth. Stiching not supported in this example" << std::endl;
	}

	// Query the recomended sample rate for the desired measurements
	//
	// Note that all analysis library parameters that impact the sample rate required must be set before querying, namely:
	// Analysis Mode, Channel BW, Measurement Span, Digitiser Span.
	double analysisSampleRate;
	status = afWlanDll_GetRecommendedSamplingFreq(wlanObj, measurements, &analysisSampleRate); // query recommended sample rate
	ErrorCheckWLAN(wlanObj,status);

	status = afWlanDll_SamplingFreq_Set(wlanObj, analysisSampleRate); // set actual sample rate that will be used for capture
	ErrorCheckWLAN(wlanObj,status);

	// Finally, query the number of stitches required.
	//
	// Note that this step is very important for spectrum measurements and must be performed even if you already know the number
	// of stitches that will be required, because this function also performs some internal initialisation of the spectrum measurements.
	long stitches;
	status = afWlanDll_Spectrum_NumStitches_Get(wlanObj, &stitches);
	ErrorCheckWLAN(wlanObj,status);

	// Perform the analysis.
	status = afWlanDll_AnalyseIQ(wlanObj, measurements, iBuffer, qBuffer, NumSamples,0,NumSamples,Tag,Key);
	ErrorCheckWLAN(wlanObj,status);

	long pos = -1;
	status = afWlanDll_Burst_Position_Get(wlanObj, &pos);
	ErrorCheckWLAN(wlanObj,status);

	long len = -1;
	status = afWlanDll_Burst_Length_Get(wlanObj, &len);
	ErrorCheckWLAN(wlanObj,status);

	float power = -1;
	status = afWlanDll_BurstProfile_AveragePower_Get(wlanObj, &power);
	ErrorCheckWLAN(wlanObj,status);

	float evm;
	status = afWlanDll_Evm_Rms_Get(wlanObj, &evm);
	ErrorCheckWLAN(wlanObj,status);

	long mcs = -1;
	status = afWlanDll_MCS_Get(wlanObj, &mcs);
	ErrorCheckWLAN(wlanObj,status);

	double obw;
	status = afWlanDll_OccupiedBandwidth_Get(wlanObj, &obw);
	ErrorCheckWLAN(wlanObj,status);

	std::cout << "\nResults" << std::endl;
	std::cout << "-------\n" << std::endl;
	std::cout << "Burst Power\t: " << power << " dBm" << std::endl;
	std::cout << "EVM\t\t: " << evm << " %" << std::endl;
	std::cout << "MCS\t\t: " << mcs << std::endl;
	std::cout << "OBW\t\t: " << obw << " Hz" << std::endl;


	// Destroy the WLAN object.
	CloseWLAN(wlanObj);
}

//****************************************************************************** 
// Function          : _tmain
// Globals Used      : None
// Globals Modified  : None
// Description       : Console App entry point.
//******************************************************************************
int _tmain(int argc, _TCHAR* argv[])
{
	ViSession vsg = VI_NULL;
	ViSession vsa = VI_NULL;
	std::vector<ViReal64> I;
	std::vector<ViReal64> Q;
	ViReal64 sampleRate = 80e6; // 80 MHz.
	ViReal64 frequency	= 1e9;	// 1 GHz
	ViReal64 dwellTime = 1e-3; // 1 ms.
	ViReal64 level		= -20;	// -20 dBm
	ViReal64 referenceLevel = -20; // 20 dBm
	ViReal64 headRoom = 16; // 16 dB of headroom.
	int myChoice = 0;

	int numberOfSamples = (int)(dwellTime * sampleRate);
	float *iData = new float[numberOfSamples];
	float *qData = new float[numberOfSamples];
	double key;
	double tag;

	std::cout << "VST Example 7 - Analyse WLAN 802.11AC Using WLAN Library with Key and Tag" << std::endl;
	std::cout << "-------------------------------------------------------------------------" << std::endl;
	std::cout << std::endl;
	// Boot and configure the RFSG.
	std::cout << "Booting and configuring RFSG." << std::endl;
	vsg = BootRFSG();
	std::cout << "Booting and configuring RFSA." << std::endl;
	vsa = BootRFSA();
	std::cout << std::endl;
	std::cout << "********** Main Menu **********" << std::endl;
	std::cout << "(1): Playback .aiq waveform and Analyse WLAN 802.11AC signal using Key and Tag." << std::endl;		
	std::cout << "(2): Exit" << std::endl;
	
	do
	{
		std::cin >> myChoice;

		switch(myChoice)
		{
		case 1:		
			PlayAiqWaveform(vsg,"WLAN_11AC_40MHz_MCS7.aiq",frequency,level);

			// Capture some IQ and get the key and tag.
			Capture(vsa, frequency, referenceLevel + headRoom, numberOfSamples, sampleRate, iData, qData,&key,&tag);

			// Analyse the WLAN Signal
			WlanAnalysis(vsa, numberOfSamples, sampleRate,iData, qData,key, tag);
			break;			
		case 2:
			break;
		default:
			std::cout << "ERROR! You have selected an invalid choice." << std::endl;
			break;
		}	
		std::cout << "********** Main Menu **********" << std::endl;
		std::cout << "(1): Playback .aiq waveform and Analyse WLAN 802.11AC signal using Key and Tag." << std::endl;		
		std::cout << "(2): Exit" << std::endl;
	}while(myChoice !=2);
	
	// Cleanup

	I.clear();
	Q.clear();

	CloseRFSG(vsg);
	CloseRFSA(vsa);
	

	return 0;
}

